@@ -40,28 +40,31 @@ hscolour :: Output      -- ^ Output format.
          -> String      -- ^ Coloured Haskell source code.
 hscolour output pref anchor partial title False =
         (if partial then id else top'n'tail output title) .
-        hscolour' output pref anchor
+        hscolour' output pref anchor 1
 hscolour output pref anchor partial title True  =
         (if partial then id else top'n'tail output title) .
-        concatMap chunk . joinL . classify . inlines
+        concat . chunk 1 . joinL . classify . inlines
   where
-    chunk (Code c) = hscolour' output pref anchor c
-    chunk (Lit c)  = c
+    chunk _        []     = []
+    chunk n (Code c: cs)  = hscolour' output pref anchor n c
+                              : chunk (n + length (lines c)) cs
+    chunk n (Lit c:  cs)  = c : chunk n cs
 
 -- | The actual colourising worker, despatched on the chosen output format.
 hscolour' :: Output      -- ^ Output format.
           -> ColourPrefs -- ^ Colour preferences (for formats that support them)
           -> Bool        -- ^ Whether to include anchors.
+          -> Int         -- ^ Starting line number (for line anchors)
           -> String      -- ^ Haskell source code.
           -> String      -- ^ Coloured Haskell source code.
-hscolour' TTY       pref _      = TTY.hscolour     pref
-hscolour' (TTYg tt) pref _      = TTY.hscolourG tt pref
-hscolour' MIRC      pref _      = MIRC.hscolour    pref
-hscolour' LaTeX     pref _      = LaTeX.hscolour   pref
-hscolour' HTML      pref anchor = HTML.hscolour    pref anchor
-hscolour' CSS       _    anchor = CSS.hscolour          anchor
-hscolour' ICSS      pref anchor = ICSS.hscolour    pref anchor
-hscolour' ACSS      _    anchor = ACSS.hscolour         anchor
+hscolour' TTY       pref _      _ = TTY.hscolour     pref
+hscolour' (TTYg tt) pref _      _ = TTY.hscolourG tt pref
+hscolour' MIRC      pref _      _ = MIRC.hscolour    pref
+hscolour' LaTeX     pref _      _ = LaTeX.hscolour   pref
+hscolour' HTML      pref anchor n = HTML.hscolour    pref anchor n
+hscolour' CSS       _    anchor n = CSS.hscolour          anchor n
+hscolour' ICSS      pref anchor n = ICSS.hscolour    pref anchor n
+hscolour' ACSS      _    anchor n = ACSS.hscolour         anchor n
 
 -- | Choose the right headers\/footers, depending on the output format.
 top'n'tail :: Output           -- ^ Output format
@@ -96,15 +99,22 @@ inlines s = lines' s id
 classify ::  [String] -> [Lit]
 classify []             = []
 classify (x:xs) | "\\begin{code}"`isPrefixOf`x
-                        = Lit x: allProg xs
-   where allProg []     = []  -- Should give an error message,
-                              -- but I have no good position information.
-         allProg (x:xs) | "\\end{code}"`isPrefixOf`x
-                        = Lit x: classify xs
-         allProg (x:xs) = Code x: allProg xs
+                        = Lit x: allProg "code" xs
+classify (x:xs) | "\\begin{spec}"`isPrefixOf`x
+                        = Lit x: allProg "spec" xs
 classify (('>':x):xs)   = Code ('>':x) : classify xs
 classify (x:xs)         = Lit x: classify xs
 
+
+allProg name  = go 
+  where
+    end       = "\\end{" ++ name ++ "}"
+    go []     = []  -- Should give an error message,
+                    -- but I have no good position information.
+    go (x:xs) | end `isPrefixOf `x
+              = Lit x: classify xs
+    go (x:xs) = Code x: go xs
+
 -- | Join up chunks of code\/comment that are next to each other.
 joinL :: [Lit] -> [Lit]
 joinL []                  = []
